<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
  xmlns:template="http://www.boddie.org.uk/ns/xmltools/template">

  <xsl:output indent="yes"/>
  <xsl:namespace-alias stylesheet-prefix="axsl" result-prefix="xsl"/>



  <!-- Match the document itself. -->

  <xsl:template match="/">
    <axsl:stylesheet version="1.0">

      <axsl:output indent="yes"/>
      <!-- NOTE: Hard-coded doctypes to hopefully satisfy JavaScript code. -->
      <!-- doctype-public="-//W3C//DTD XHTML 1.1//EN"
        doctype-system="http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd" -->

      <axsl:template match="/">

        <!-- Include the remaining attributes. -->
        <xsl:apply-templates select="@*"/>

        <!-- Process the elements. -->
        <xsl:apply-templates select="*"/>

      </axsl:template>
    </axsl:stylesheet>
  </xsl:template>



  <!-- Match element references. -->

  <xsl:template match="*[@template:element]">
    <xsl:call-template name="enter-element">
      <xsl:with-param name="other-elements" select="@template:element"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="enter-element">
    <xsl:param name="other-elements"/>
    <xsl:variable name="first-element" select="substring-before($other-elements, ',')"/>
    <xsl:variable name="remaining-elements" select="substring-after($other-elements, ',')"/>
    <xsl:choose>
      <xsl:when test="$first-element = ''">
        <xsl:call-template name="next-element">
          <xsl:with-param name="first-element" select="$other-elements"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="next-element">
          <xsl:with-param name="first-element" select="$first-element"/>
          <xsl:with-param name="remaining-elements" select="$remaining-elements"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="next-element">
    <xsl:param name="first-element"/>
    <xsl:param name="remaining-elements"/>
    <axsl:for-each select="{$first-element}">
      <xsl:choose>
        <xsl:when test="$remaining-elements = ''">
          <xsl:call-template name="special-attributes"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="enter-element">
            <xsl:with-param name="other-elements" select="$remaining-elements"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </axsl:for-each>
  </xsl:template>



  <!-- Match special expression attributes. -->

  <xsl:template match="*[not(@template:element) and (@template:attribute or @template:value or @template:expr)]">
    <xsl:call-template name="special-attributes"/>
  </xsl:template>

  <xsl:template name="special-attributes">
    <xsl:choose>
      <xsl:when test="@template:effect = 'replace'">
        <xsl:call-template name="special-values"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy>
          <xsl:apply-templates select="@*"/>
          <xsl:if test="@template:expr and @template:expr-attr">
            <axsl:if test="{@template:expr}">
              <axsl:attribute name="{@template:expr-attr}"><xsl:value-of select="@template:expr-attr"/></axsl:attribute>
            </axsl:if>
          </xsl:if>
          <xsl:call-template name="special-values"/>
        </xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="special-values">
    <xsl:choose>
      <xsl:when test="@template:attribute">
        <axsl:choose>
          <axsl:when test="@{@template:attribute}">
            <axsl:variable name="this-name"><xsl:value-of select="@template:attribute"/></axsl:variable>
            <axsl:variable name="this-value" select="@{@template:attribute}"/>
            <xsl:call-template name="special-value"/>
          </axsl:when>
          <axsl:otherwise>
            <axsl:variable name="this-name"><xsl:value-of select="@template:attribute"/></axsl:variable>
            <axsl:variable name="this-value"></axsl:variable>
            <xsl:call-template name="special-value"/>
          </axsl:otherwise>
        </axsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="special-value"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="special-value">
    <xsl:choose>
      <xsl:when test="@template:value">
        <axsl:value-of select="{@template:value}"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="node()"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- Remove template elements. -->

  <xsl:template match="@template:element|@template:attribute|@template:value|@template:expr|@template:expr-attr|@template:effect">
  </xsl:template>



  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
